/*
 * YDFWTA.java
 *
 * Created on 18. Mai 2000, 09:53
 */
 
package YARB.Filter;
import YARB.Interfaces.*;
import javax.swing.*;
import YARB.Dialogs.*;

/** 
 * YDFWTA - YARB Data Filter: The winner takes it all
 * This data filter processes all pattern and units and gives for every pattern the 
 * index number of the greatest unit. (numbers range from 0 to numbOfXXXUnits()-1)
 * @author  Thomas
 * @version 
 */
public class YDFWTA extends Object implements IDataFilter
{
  
  float      m_InBuffer[][];
  float      m_OutBuffer[][];
  int        m_nNumbOfPattern;
  int        m_nNumbOfInUnits;
  int        m_nNumbOfOutUnits;
   
  int        m_VarInDimension;
  int        m_MaxVarInDim[];
  int        m_VarOutDimension;
  int        m_MaxVarOutDim[];
  
  boolean    m_bProcessInput;
  boolean    m_bProcessOutput;
  
  IDataProvider m_DP;  // the previous provider

  /** Creates new YDFWTA */
  public YDFWTA() {
    m_DP=null;
    m_InBuffer=null;
    m_OutBuffer=null;
    m_MaxVarInDim=null;
    m_MaxVarOutDim=null;
    m_nNumbOfPattern=0;
    m_nNumbOfInUnits=0;
    m_nNumbOfOutUnits=0;
    m_VarInDimension=0;
    m_VarOutDimension=0;
    m_bProcessInput=true;
    m_bProcessOutput=true;

  }
  
  /** Sets the previour data provider.
   * This coudl be another data filter or a file source.
   */
  public void setPreviousProvider(IDataProvider DP) {
    m_DP=DP;
  }
  
  /** Starts the data processing task.
   */
  public boolean ProcessData() throws IDataProvider.DataProcessingError
  {
    if(m_DP==null) return false;
    
    //** Don't forget: You have to call the ProcessData method of your predecessor
    m_DP.ProcessData();
    
    //** Now the data can be read and processed:
    
    // create the new Array of Pattern
    if(m_bProcessInput){
         m_InBuffer= new float[m_DP.numbOfPatterns()][1];
    }
    else{
        m_InBuffer= new float[m_DP.numbOfPatterns()][m_DP.numbOfInputUnits()];
    };
    
    if(m_bProcessOutput){
         m_OutBuffer= new float[m_DP.numbOfPatterns()][1];
    }
    else{
        m_OutBuffer= new float[m_DP.numbOfPatterns()][m_DP.numbOfOutputUnits()];
    };
    
    // Copy all input Pattern
    if(m_DP.numbOfInputUnits()>0)
    {
      if(m_bProcessInput) // use WTA for this pattern?
      {
        for(int i =0; i < m_DP.numbOfPatterns(); i++)
        {
          m_InBuffer[i][0]=getIndexOfMaxInputUnit(m_DP,i);
        };
        m_nNumbOfInUnits=1;
      }
      else // Only a copy action...
      {
        for(int i =0; i < m_DP.numbOfPatterns(); i++)
        {
          for(int k=0; k< m_DP.numbOfInputUnits(); k++)
          {
            m_InBuffer[i][k]=m_DP.getInputData(i,k);
          };
        };
        m_nNumbOfInUnits=m_DP.numbOfInputUnits();
      };
    }
    else // no input units
    {  
      m_nNumbOfInUnits=0;
    };

    // Copy all output Pattern
    if(m_DP.numbOfOutputUnits()>0)
    {
      if(m_bProcessOutput)  // use WTA for this pattern?
      {
        for(int i =0; i < m_DP.numbOfPatterns(); i++)
        {
          m_OutBuffer[i][0]=getIndexOfMaxOutputUnit(m_DP,i);
        };
        m_nNumbOfOutUnits=1;
      }
      else // only pattern copy
      {
        for(int i =0; i < m_DP.numbOfPatterns(); i++)
        {
          for(int k=0; k< m_DP.numbOfOutputUnits(); k++)
          {
            m_OutBuffer[i][k]=m_DP.getOutputData(i,k);
          };
        };
        m_nNumbOfOutUnits=m_DP.numbOfOutputUnits();
      };
    }
    else // no output pattern...
    {  
      m_nNumbOfOutUnits=0;
    };
      
   // Set state information:
    m_nNumbOfPattern=m_DP.numbOfPatterns();
    m_VarInDimension= m_DP.numbOfVariableInputDimensions();
    m_VarOutDimension= m_DP.numbOfVariableOutputDimensions();
      
    if(m_VarInDimension>0)
    {
        m_MaxVarInDim= new int[m_VarInDimension];
        for(int j=0; j<m_VarInDimension;j++)
        {
          m_MaxVarInDim[j]=m_DP.getMaxInputDimesion(j);
        };
    };
      
    if(m_VarOutDimension>0)
    {
        m_MaxVarOutDim= new int[m_VarOutDimension];
        for(int j=0; j<m_VarOutDimension;j++)
        {
          m_MaxVarOutDim[j]=m_DP.getMaxOutputDimesion(j);
        };
    };
    
    
    
    //** Data processing finished.
    
    //** Don't forget: Call the FreeData method of your predecessor!
    m_DP.FreeData();
    
    
    // Everything O.K.
    return true;
    
  }
  
  private int getIndexOfMaxInputUnit(IDataProvider DP, int nPattern)
  {
    if (DP==null) return -1;
    
    int m=0;
    float f= DP.getInputData(nPattern,0);
    
    for(int i=0; i<DP.numbOfInputUnits();i++)
    {
      if(DP.getInputData(nPattern,i) > f) 
      {
        m=i;
        f=DP.getInputData(nPattern,i);
      };
    };
    
    return m;    
  }

  private int getIndexOfMaxOutputUnit(IDataProvider DP, int nPattern)
  {
    if (DP==null) return -1;
    
    int m=0;
    float f= DP.getOutputData(nPattern,0);
    
    for(int i=0; i<DP.numbOfOutputUnits();i++)
    {
      if(DP.getOutputData(nPattern,i) > f)
      {
        m=i;
        f=DP.getOutputData(nPattern,i);
      };
      
    };
    
    return m;    
  }
  
  
  /** Frees the internal buffers
   */
  public boolean FreeData() 
  {
    if(m_DP==null) return false;

    m_InBuffer=null;
    m_OutBuffer=null;
    m_MaxVarInDim=null;
    m_MaxVarOutDim=null;
    m_nNumbOfPattern=0;
    m_nNumbOfInUnits=0;
    m_nNumbOfOutUnits=0;
    m_VarInDimension=0;
    m_VarOutDimension=0;
    
    return true;
  }
  
  /** This function forces the object to save its configuration data to a file
   */
  public boolean SaveConfiguration(IConfigurationWriter Cfg) throws IConfigurationWriter.ECfgWriterError {
    if(Cfg==null) return false;
    
    Cfg.addTitle(getName());
    Cfg.addBoolean(m_bProcessInput);
    Cfg.addBoolean(m_bProcessOutput);
    
    return true;
  }
  
  /** This function forces the object to read a configuration from a file
   */
  public boolean LoadConfiguration(IConfigurationReader Cfg) throws IConfigurationReader.ECfgReaderError {
    if(Cfg==null) return false;
    
    setName(Cfg.getTitle());
    m_bProcessInput=Cfg.getBoolean();
    m_bProcessOutput=Cfg.getBoolean();
    
    return true;
  } 

  /** Can the object be configured by the user
   */
  public boolean IsConfigurable() {
    return true;
  }
  
  /** Shows the configuration dialog provided by the object
   */
  public boolean ShowConfigurationDialog(JFrame ParentFrame) 
  {
    YFilterNameChooser dlg= new YFilterNameChooser(ParentFrame, "YARB Filter Properties",true,getName());
    
    dlg.setProcessInputPattern(m_bProcessInput);
    dlg.setProcessOutputPattern(m_bProcessOutput);
    
    dlg.show();
    
    setName(dlg.getName());
    m_bProcessInput=dlg.getProcessInputPattern();
    m_bProcessOutput=dlg.getProcessOutputPattern();
    
    return true;  
  }
  

  /**
  * Contains the assigned name string.
  */
  private String  m_strName;
  
   /** Sets the Name of the object
   * This is a user-defined name. Has nothig to do with the toName-Function
   */
  public void setName(String strName) {
    m_strName=strName;
  }
 
  /** Gets the user defined name of the object.
   * This is a user-defined name. Has nothig to do with the toName-Function
   */
  public String getName() {
    return m_strName;
  }
  
  /** Gets the number of pattern.
   */
  public int numbOfPatterns() {
    return m_nNumbOfPattern;
  }
  
  /** Gets the number of input units.
   */
  public int numbOfInputUnits() {
    return m_nNumbOfInUnits;
  }
  
  /** Gets the number of Output Units
   */
  public int numbOfOutputUnits() {
    return m_nNumbOfOutUnits;
  }
  
  /** Gets the number of variable Input dimensions
   */
  public int numbOfVariableInputDimensions() {
    return m_VarInDimension;
  }
  
  /** Gets the number of variable Output dimensions
   */
  public int numbOfVariableOutputDimensions() {
    return m_VarOutDimension;
  }
  
  /** Gets the maximum input dimensions range for the given variable input dimension
  */
  public int getMaxInputDimesion(int nVarInputDimension) 
  {
    // Check array bounds:
    if(nVarInputDimension>=m_VarInDimension || nVarInputDimension < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getMaxInputDimension(int); Index: " + nVarInputDimension + "; Max: " + m_VarInDimension + ";)";
      System.out.println(msg);
      return 0;
    };
    // Everything O.K.
    return m_MaxVarInDim[nVarInputDimension];
  }
  
  /** Gets the maximum output dimensions range for the given variable output dimension
   */
  public int getMaxOutputDimesion(int nVarOutputDimension)
  {
    // Check array bounds:
    if(nVarOutputDimension>=m_VarOutDimension || nVarOutputDimension < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getMaxOutputDimension(int); Index: " + nVarOutputDimension + "; Max: " + m_VarOutDimension + ";)";
      System.out.println(msg);
      return 0;
    };
    // Everything O.K.
    return m_MaxVarOutDim[nVarOutputDimension];
  }
  
  /** Gets a input data field, inthe given pattern and specified by the give unit.
   */
  public float getInputData(int nPattern,int nUnit) 
  {
    // Check array Bounds:
    if(nPattern>=m_nNumbOfPattern || nPattern < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getInputData(int,int); Index: " + nPattern + "; Max: " + m_nNumbOfPattern + ";)";
      System.out.println(msg);
      return 0;
    };
    if(nUnit>=m_nNumbOfInUnits || nUnit < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getInputData(int,int); Index: " + nUnit + "; Max: " + m_nNumbOfInUnits + ";)";
      System.out.println(msg);
      return 0;
    };
    // Bounds OK, return value
    return m_InBuffer[nPattern][nUnit];
  }
  
  
  /** Gets a output data field, inthe given pattern and specified by the give unit.
   */
  public float getOutputData(int nPattern,int nUnit) 
  {
    // Check array Bounds:
    if(nPattern>=m_nNumbOfPattern || nPattern < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getOutputData(int,int); Index: " + nPattern + "; Max: " + m_nNumbOfPattern + ";)";
      System.out.println(msg);
      return 0;
    };
    if(nUnit>=m_nNumbOfOutUnits || nUnit < 0)
    {
      String msg="ERROR: Array bound exceeded! (Object: " + this.toString() + "; Method: getOutputData(int,int); Index: " + nUnit + "; Max: " + m_nNumbOfOutUnits + ";)";
      System.out.println(msg);
      return 0;
    };
    // Bounds OK, return value
    return m_OutBuffer[nPattern][nUnit];
  }
}